@@ -13,6 +13,7 @@ import (
 
 	"tailscale.com/ipn"
 	"tailscale.com/ipn/ipnserver"
+	"tailscale.com/ipn/store/mem"
 	"tailscale.com/net/tsdial"
 	"tailscale.com/safesocket"
 	"tailscale.com/wgengine"
@@ -27,7 +28,7 @@ func TestRunMultipleAccepts(t *testing.T) {
 	td := t.TempDir()
 	socketPath := filepath.Join(td, "tailscale.sock")
 
-	logf := func(format string, args ...interface{}) {
+	logf := func(format string, args ...any) {
 		format = strings.TrimRight(format, "\n")
 		println(fmt.Sprintf(format, args...))
 		t.Logf(format, args...)
@@ -51,7 +52,7 @@ func TestRunMultipleAccepts(t *testing.T) {
 		}
 	}
 
-	logTriggerTestf := func(format string, args ...interface{}) {
+	logTriggerTestf := func(format string, args ...any) {
 		logf(format, args...)
 		if strings.HasPrefix(format, "Listening on ") {
 			connect()
@@ -66,7 +67,7 @@ func TestRunMultipleAccepts(t *testing.T) {
 
 	opts := ipnserver.Options{}
 	t.Logf("pre-Run")
-	store := new(ipn.MemoryStore)
+	store := new(mem.Store)
 
 	ln, _, err := safesocket.Listen(socketPath, 0)
 	if err != nil {
