@@ -40,7 +40,7 @@ func (h UDP6Header) Marshal(buf []byte) error {
 	binary.BigEndian.PutUint16(buf[46:48], 0) // blank checksum
 
 	// UDP checksum with IP pseudo header.
-	h.IP6Header.marshalPseudo(buf)
+	h.IP6Header.marshalPseudo(buf, ipproto.UDP)
 	binary.BigEndian.PutUint16(buf[46:48], ip4Checksum(buf[:]))
 
 	h.IP6Header.Marshal(buf)
