@@ -9,6 +9,8 @@ import (
 	"context"
 	"errors"
 	"expvar"
+	"fmt"
+	"io"
 	"net"
 	"net/http"
 	"net/http/httptest"
@@ -241,7 +243,7 @@ func TestStdHandler(t *testing.T) {
 	for _, test := range tests {
 		t.Run(test.name, func(t *testing.T) {
 			var logs []AccessLogRecord
-			logf := func(fmt string, args ...interface{}) {
+			logf := func(fmt string, args ...any) {
 				if fmt == "%s" {
 					logs = append(logs, args[0].(AccessLogRecord))
 				}
@@ -378,19 +380,19 @@ func TestVarzHandler(t *testing.T) {
 		{
 			"func_float64",
 			"counter_x",
-			expvar.Func(func() interface{} { return float64(1.2) }),
+			expvar.Func(func() any { return float64(1.2) }),
 			"# TYPE x counter\nx 1.2\n",
 		},
 		{
 			"func_float64_gauge",
 			"gauge_x",
-			expvar.Func(func() interface{} { return float64(1.2) }),
+			expvar.Func(func() any { return float64(1.2) }),
 			"# TYPE x gauge\nx 1.2\n",
 		},
 		{
 			"func_float64_untyped",
 			"x",
-			expvar.Func(func() interface{} { return float64(1.2) }),
+			expvar.Func(func() any { return float64(1.2) }),
 			"x 1.2\n",
 		},
 		{
@@ -466,9 +468,15 @@ foo_AUint16 65535
 		{
 			"func_returning_int",
 			"num_goroutines",
-			expvar.Func(func() interface{} { return 123 }),
+			expvar.Func(func() any { return 123 }),
 			"num_goroutines 123\n",
 		},
+		{
+			"var_that_exports_itself",
+			"custom_var",
+			promWriter{},
+			"custom_var_value 42\n",
+		},
 	}
 	for _, tt := range tests {
 		t.Run(tt.name, func(t *testing.T) {
@@ -531,6 +539,16 @@ type expvarAdapter struct {
 
 func (expvarAdapter) String() string { return "{}" } // expvar JSON; unused in test
 
-func (a expvarAdapter) PrometheusMetricsReflectRoot() interface{} {
+func (a expvarAdapter) PrometheusMetricsReflectRoot() any {
 	return a.st
 }
+
+type promWriter struct{}
+
+func (promWriter) WritePrometheus(w io.Writer, prefix string) {
+	fmt.Fprintf(w, "%s_value 42\n", prefix)
+}
+
+func (promWriter) String() string {
+	return ""
+}
