@@ -8,6 +8,7 @@ import (
 	"testing"
 
 	"github.com/google/go-cmp/cmp"
+	"tailscale.com/tstest"
 )
 
 func TestParse(t *testing.T) {
@@ -39,11 +40,11 @@ func TestParse(t *testing.T) {
 		if diff := cmp.Diff(gotParsed, test.parsed); diff != "" {
 			t.Errorf("parse(%q) diff (-got+want):\n%s", test.version, diff)
 		}
-		n := int(testing.AllocsPerRun(1000, func() {
+		err := tstest.MinAllocsPerRun(t, 0, func() {
 			gotParsed, got = parse(test.version)
-		}))
-		if n != 0 {
-			t.Errorf("parse(%q) allocs = %d; want 0", test.version, n)
+		})
+		if err != nil {
+			t.Errorf("parse(%q): %v", test.version, err)
 		}
 	}
 }
