@@ -70,6 +70,16 @@ func MatchesFromFilterRules(pf []tailcfg.FilterRule) ([]Match, error) {
 				})
 			}
 		}
+		for _, cm := range r.CapGrant {
+			for _, dstNet := range cm.Dsts {
+				for _, cap := range cm.Caps {
+					m.Caps = append(m.Caps, CapMatch{
+						Dst: dstNet,
+						Cap: cap,
+					})
+				}
+			}
+		}
 
 		mm = append(mm, m)
 	}
@@ -114,8 +124,7 @@ func parseIPSet(arg string, bits *int) ([]netaddr.IPPrefix, error) {
 		return []netaddr.IPPrefix{pfx}, nil
 	}
 	if strings.Count(arg, "-") == 1 {
-		i := strings.Index(arg, "-")
-		ip1s, ip2s := arg[:i], arg[i+1:]
+		ip1s, ip2s, _ := strings.Cut(arg, "-")
 		ip1, err := netaddr.ParseIP(ip1s)
 		if err != nil {
 			return nil, err
