@@ -8,6 +8,7 @@ import (
 	"runtime"
 	"testing"
 
+	"gvisor.dev/gvisor/pkg/refs"
 	"inet.af/netaddr"
 	"tailscale.com/net/packet"
 	"tailscale.com/net/tsdial"
@@ -21,7 +22,7 @@ import (
 func TestInjectInboundLeak(t *testing.T) {
 	tunDev := tstun.NewFake()
 	dialer := new(tsdial.Dialer)
-	logf := func(format string, args ...interface{}) {
+	logf := func(format string, args ...any) {
 		if !t.Failed() {
 			t.Logf(format, args...)
 		}
@@ -74,3 +75,12 @@ func getMemStats() (ms runtime.MemStats) {
 	runtime.ReadMemStats(&ms)
 	return
 }
+
+func TestNetstackLeakMode(t *testing.T) {
+	// See the comments in init(), and/or in issue #4309.
+	// Influenced by an envknob that may be useful in tests, so just check that
+	// it's not the oddly behaving zero value.
+	if refs.GetLeakMode() == 0 {
+		t.Fatalf("refs.leakMode is 0, want a non-zero value")
+	}
+}
